console.log('Target origin:', 'https://demo1.cmsbuddy.chat');

if (window.self !== window.top) {
    let highlightOverlay = null; // Overlay for highlighting elements

    let selectedElement = null;
    function getElementScrollTop(element) {
        if (!element) return 0;

        const rect = element.getBoundingClientRect();
        return rect.top + window.scrollY;
    }


    // Listen for messages from the parent
    window.addEventListener('message', (event) => {
        // // Validate the message origin for security
        // if (event.origin !== 'http://localhost:3000') { // Replace with your ReactJS app's origin
        //     console.warn('Blocked message from untrusted origin:', event.origin);
        //     return;
        // }

        // Handle the restoreScroll message
        if (event.data.type === 'restoreScroll') {
            const { scrollTop } = event.data.data;
            window.scrollTo({ top: scrollTop, behavior: 'smooth' });
        }

        if (event.data.type === 'updateElement') {
            const { content } = event.data.data; // Match the property sent in postMessage

            if (selectedElement) {
                // Update the selected DOM element
                selectedElement.innerHTML = content;
            } else {
                console.warn('No element selected for update');
            }
        }
        // Add highlight to the selected element
        if (event.data.type === 'addHighlight') {
            // Remove the highlight class from all elements first
            document.querySelectorAll('.cmsbuddy_highlight').forEach((element) => {
                element.classList.remove('cmsbuddy_highlight');
            });

            // Add the highlight class to the selected element
            if (selectedElement) {
                selectedElement.classList.add('cmsbuddy_highlight');
            } else {
                console.warn('No selected element to highlight.');
            }
        }

        // Remove all highlights
        if (event.data.type === 'removeHighlight') {
            // Remove the highlight class from all elements
            document.querySelectorAll('.cmsbuddy_highlight').forEach((element) => {
                selectedElement.classList.remove('cmsbuddy_highlight');
            });
        }

        // Handle the "detectElement" message
        if (event.data.type === 'detectElement') {
            const { htmlString } = event.data.data; // HTML string sent from ReactJS

            // Detect the element by its HTML
            const selectedElement = detectElementByHTML(htmlString);

            if (selectedElement) {
                // Optional: Add highlight or perform other actions on the detected element
                selectedElement.classList.add('cmsbuddy_highlight');

            } else {
                console.warn('No matching element found.');

                // Send a failure response back to ReactJS
                event.source.postMessage(
                    {
                        type: 'elementDetected',
                        data: {
                            success: false,
                            error: 'No matching element found.'
                        }
                    },
                    event.origin
                );
            }
        }

        // Handle the "detectElement" message
        if (event.data.type === 'getContent') {
            //get element next of <header> tag
            selectedElement = document.getElementById('cmsbuddy_review_content');
            const content = selectedElement.innerHTML;
            //return content to react
            event.source.postMessage(
                {
                    type: 'returnContent',
                    data: {
                        content
                    }
                },
                event.origin
            );
        }



    });

    // Report the current scroll position to the parent
    const reportScrollPosition = () => {
        const scrollTop = document.documentElement.scrollTop || window.scrollY;
        window.parent.postMessage(
            {
                type: 'scrollPosition',
                data: { scrollTop },
            },
            '*'
        );
    };

    // Listen for scroll events in the iframe
    window.addEventListener('scroll', reportScrollPosition);


    // Add hover listener to highlight elements
    document.addEventListener('mouseover', (event) => {
        const target = event.target;

        // Skip if hovering over the overlay itself
        if (target === highlightOverlay) return;

        // Create or update the highlight overlay
        if (!highlightOverlay) {
            highlightOverlay = document.createElement('div');
            document.body.appendChild(highlightOverlay);
            styleHighlightOverlay(highlightOverlay);
        }

        // Get the target element's bounding rectangle
        const rect = target.getBoundingClientRect();

        // Position and size the overlay to match the hovered element
        highlightOverlay.style.top = `${rect.top + window.scrollY}px`;
        highlightOverlay.style.left = `${rect.left + window.scrollX}px`;
        highlightOverlay.style.width = `${rect.width}px`;
        highlightOverlay.style.height = `${rect.height}px`;
        highlightOverlay.style.display = 'block';
    });

    // Remove the highlight overlay on mouseout
    document.addEventListener('mouseout', () => {
        if (highlightOverlay) {
            highlightOverlay.style.display = 'none';
        }
    });

    // Handle link clicks
    document.addEventListener('click', (event) => {
        const target = event.target;

        // Check if the clicked element is a link
        if (target.tagName === 'A') {
            const clickedUrl = target.href;

            console.log('Link Clicked:', clickedUrl);
            selectedElement = document.getElementById('cmsbuddy_review_content');
            const content = selectedElement.innerHTML;
            // Send the clicked URL to the parent
            window.parent.postMessage(
                {
                    type: 'linkClick',
                    data: { href: clickedUrl, content: content },
                },
                '*'
            );

            // Prevent default navigation behavior
           //event.preventDefault();
        }
    });

    // Handle text selection and element HTML on double-click
    document.addEventListener('dblclick', (event) => {
        const target = event.target;
        selectedElement = event.target;
        const scrollTop = getElementScrollTop(target);

        // Get the full HTML and plain text of the clicked element
        let fullHtml = target.innerHTML; // Full HTML of the clicked element

        // Trim newlines and spaces only from the beginning and end
        fullHtml = fullHtml
            .replace(/^(\s*|\r\n|\n|\r)/, '') // Remove newlines/spaces at the start
            .replace(/(\s*|\r\n|\n|\r)$/, '') // Remove newlines/spaces at the end
            .trim(); // Ensure any trailing whitespace is removed

        const selectedText = target.textContent.trim(); // Full plain text of the clicked element

        // Send the data to the parent or handle it further
        window.parent.postMessage(
            {
                type: 'textSelect',
                data: {
                    selectedText, // Include the selected text
                    fullHtml, // Include the trimmed full HTML of the element
                    scrollTop
                },
            },
            '*'
        );
    });

    // Utility: Style the highlight overlay
    function styleHighlightOverlay(overlay) {
        overlay.style.position = 'absolute';
        overlay.style.pointerEvents = 'none'; // Ignore pointer events
        overlay.style.backgroundColor = 'rgba(0, 123, 255, 0.2)'; // Highlight color
        overlay.style.border = '2px solid rgba(0, 123, 255, 0.8)'; // Border color
        overlay.style.zIndex = '9999'; // Ensure it's above all other elements
        overlay.style.display = 'none'; // Initially hidden
    }
} else {
    console.warn('This script is not running inside an iframe. Exiting.');
}
