<?php

function get_all_post_types_from_db()
{
    // Retrieve post types that are shown in the admin and have UI, regardless of public visibility
    $args = array(
        'public'   => true,
        'show_ui' => true, // Includes post types with UI in the admin
    );

    // Get post types that match the criteria
    $post_types = get_post_types($args, 'names');

    return $post_types;

    // global $wpdb;

    // // Query to get all unique post types from the wp_posts table
    // $post_types = $wpdb->get_col("SELECT DISTINCT post_type FROM {$wpdb->posts}");

    // return $post_types;
}

function autobloggpt_edit_event(WP_REST_Request $request)
{
    remove_filter('content_save_pre', 'wp_filter_post_kses');
    remove_filter('the_content', 'wpautop'); // Disable wpautop temporarily

    $event_id = intval($request->get_param('post_id'));

    if (empty($event_id) || get_post_type($event_id) !== 'tribe_events') {
        return new WP_Error('invalid_event_id', 'Invalid event ID', array('status' => 400));
    }

    $title = sanitize_text_field($request->get_param('title'));
    $content = wp_kses_post($request->get_param('content'));
    $start_date = sanitize_text_field($request->get_param('start_date'));
    $end_date = sanitize_text_field($request->get_param('end_date'));
    $tags = sanitize_text_field($request->get_param('tags'));
    $image_url = esc_url_raw($request->get_param('image_url'));

    $content = remove_newlines_from_media_tags($content);
    $event_data = array(
        'ID'           => $event_id,
        'post_title'   => $title,
        'post_content' => $content,
    );

    $updated_event_id = wp_update_post($event_data);

    if (is_wp_error($updated_event_id)) {
        return new WP_Error('event_update_failed', 'Failed to update event', array('status' => 500));
    }

    if (!empty($start_date)) {
        update_post_meta($updated_event_id, '_EventStartDate', $start_date);
        update_post_meta($updated_event_id, '_EventStartDateUTC', get_gmt_from_date($start_date));
    }
    if (!empty($end_date)) {
        update_post_meta($updated_event_id, '_EventEndDate', $end_date);
        update_post_meta($updated_event_id, '_EventEndDateUTC', get_gmt_from_date($end_date));
    }

    if (!empty($tags)) {
        $tags_array = explode(',', $tags);
        wp_set_post_tags($updated_event_id, $tags_array);
    }

    if ($image_url) {
        $attachment_id = upload_image_from_url($image_url, $updated_event_id);
        if (!is_wp_error($attachment_id)) {
            set_post_thumbnail($updated_event_id, $attachment_id);
        }
    }
    add_filter('the_content', 'wpautop'); // Re-enable wpautop
    add_filter('content_save_pre', 'wp_filter_post_kses');

    return new WP_REST_Response(array(
        'event_id'  => $updated_event_id,
        'status'    => 'updated',
        'permalink' => get_permalink($updated_event_id),
    ), 200);
}

function autobloggpt_create_event_v2(WP_REST_Request $request) {
    // Remove filters temporarily to avoid auto-formatting issues
    remove_filter('content_save_pre', 'wp_filter_post_kses');
    remove_filter('the_content', 'wpautop');

    // Retrieve request parameters
    $title = sanitize_text_field($request->get_param('title'));
    $content = $request->get_param('content');
    $tags = sanitize_text_field($request->get_param('tags'));
    $image_url = esc_url_raw($request->get_param('image_url'));

    // Set default dates for the event
    $start_date = current_time('Y-m-d H:i:s'); // Current date and time
    $end_date = date('Y-m-d H:i:s', strtotime('+1 hour', strtotime($start_date))); // +1 hour from start
    $content = remove_newlines_from_media_tags($content);

    // Create event post
    $event_data = array(
        'post_title'   => $title,
        'post_content' => $content,
        'post_status'  => 'publish',
        'post_type'    => 'tribe_events', // The Events Calendar custom post type
    );

    // Insert the post into the database
    $event_id = wp_insert_post($event_data);

    if (is_wp_error($event_id)) {
        return new WP_Error('event_creation_failed', 'Failed to create event', array('status' => 500));
    }

    // Add event metadata (start and end dates)
    update_post_meta($event_id, '_EventStartDate', $start_date);
    update_post_meta($event_id, '_EventEndDate', $end_date);
    update_post_meta($event_id, '_EventTimezone', 'Europe/Berlin');
    update_post_meta($event_id, '_EventShowMap', true);
    update_post_meta($event_id, '_EventShowMapLink', true);

    // Add tags to the event
    if (!empty($tags)) {
        $tags_array = explode(',', $tags);
        wp_set_object_terms($event_id, $tags_array, 'tribe_events_cat'); // Assign tags to the event category
    }

    // Upload and set featured image, if provided
    if ($image_url) {
        $attachment_id = upload_image_from_url($image_url, $event_id);
        if (!is_wp_error($attachment_id)) {
            set_post_thumbnail($event_id, $attachment_id);
        }
    }

    // Re-enable filters
    add_filter('the_content', 'wpautop');
    add_filter('content_save_pre', 'wp_filter_post_kses');

    // Return response with event details
    return new WP_REST_Response(array(
        'event_id'  => $event_id,
        'status'    => 'created',
        'permalink' => get_permalink($event_id),
    ), 200);
}




function autobloggpt_create_event(WP_REST_Request $request)
{

    remove_filter('content_save_pre', 'wp_filter_post_kses');
    remove_filter('the_content', 'wpautop'); // Disable wpautop temporarily

    // Get data from the request
    $title = sanitize_text_field($request->get_param('title'));
    $content = $request->get_param('content');
    $tags = sanitize_text_field($request->get_param('tags'));
    $image_url = esc_url_raw($request->get_param('image_url'));

    // Default dates for the event
    $start_date = current_time('Y-m-d H:i:s');
    $end_date = date('Y-m-d H:i:s', strtotime('+1 hour', strtotime($start_date)));
    $content = remove_newlines_from_media_tags($content);
    // Event data payload for the API
    $event_data = array(
        'title'         => $title,
        'description'   => $content,
        'start_date'    => $start_date,
        'end_date'      => $end_date,
        'timezone'      => 'Europe/Berlin',
        'show_map'      => true,
        'show_map_link' => true,
        'tags'          => $tags ? explode(',', $tags) : array(),
    );

    // API endpoint for The Events Calendar
    $url = site_url('/wp-json/tribe/events/v1/events');

    // Replace with your username and application password
    $username = 'trummerc91@gmail.com';
    $application_password = 'qGrd 0KEk Gd6B qlR1 Im7T nb1n';

    // Send API request to create the event
    $response = wp_remote_post($url, array(
        'headers' => array(
            'Authorization' => 'Basic ' . base64_encode($username . ':' . $application_password),
            'Content-Type'  => 'application/json',
        ),
        'body'    => json_encode($event_data),
        'method'  => 'POST',
    ));
    // Handle response
    if (is_wp_error($response)) {
        return new WP_Error('api_request_failed', 'Failed to create event via API', array('status' => 500));
    }

    $body = json_decode(wp_remote_retrieve_body($response), true);
    add_filter('the_content', 'wpautop'); // Re-enable wpautop
    add_filter('content_save_pre', 'wp_filter_post_kses');
    // Check if the API response indicates success
    if (isset($body['id'])) {
        $post_id = $body['id'];
        // Optionally, set the featured image if provided
        if ($image_url) {
            $attachment_id = upload_image_from_url($image_url, $body['id']);
            if (!is_wp_error($attachment_id)) {
                set_post_thumbnail($body['id'], $attachment_id);
            }
        }
        $permalink = get_permalink($post_id);
        // Return a response with the new event details
        return new WP_REST_Response(array(
            'post_id'   => $body['id'],
            'status'    => 'created',
            'permalink' => $permalink,
        ), 200);
    }

    // If the response does not contain an ID, return an error
    return new WP_Error('api_creation_failed', 'Event could not be created', array('status' => 500));
}

function autobloggpt_delete_event(WP_REST_Request $request)
{
    $event_id = intval($request->get_param('post_id'));

    if (empty($event_id) || get_post_type($event_id) !== 'tribe_events') {
        return new WP_Error('invalid_event_id', 'Invalid event ID', array('status' => 400));
    }

    $deleted = wp_delete_post($event_id, true);

    if (!$deleted) {
        return new WP_Error('event_deletion_failed', 'Failed to delete event', array('status' => 500));
    }

    return new WP_REST_Response(array('event_id' => $event_id, 'status' => 'deleted'), 200);
}

function autobloggpt_list_events(WP_REST_Request $request)
{
    $number_of_events = intval($request->get_param('number_of_events')) ?: 10;

    $events = get_posts(array(
        'numberposts' => $number_of_events,
        'post_type'   => 'tribe_events',
        'post_status' => 'publish',
    ));

    if (empty($events)) {
        return new WP_Error('no_events_found', 'No events found', array('status' => 404));
    }

    $event_list = array();
    foreach ($events as $event) {
        $event_list[] = array(
            'event_id'    => $event->ID,
            'title'       => get_the_title($event->ID),
            'description' => wp_trim_words($event->post_content, 40, '...'),
            'start_date'  => get_post_meta($event->ID, '_EventStartDate', true),
            'end_date'    => get_post_meta($event->ID, '_EventEndDate', true),
            'timezone'    => get_post_meta($event->ID, '_EventTimezone', true),
            'author'      => get_the_author_meta('display_name', $event->post_author),
            'categories'  => wp_get_post_categories($event->ID, array('fields' => 'names')),
            'tags'        => wp_get_post_tags($event->ID, array('fields' => 'names')),
            'image_url'   => get_the_post_thumbnail_url($event->ID),
            'permalink'   => get_permalink($event->ID),
        );
    }

    return new WP_REST_Response($event_list, 200);
}





function autobloggpt_create_post_by_post_type(WP_REST_Request $request)
{
    $post_type = $request->get_param('post_type');
    if ($post_type === 'tribe_events') {
        return autobloggpt_create_event_v2($request);
    } else {
        return autobloggpt_create_post($request);
    }
}

function autobloggpt_create_post(WP_REST_Request $request)
{
    remove_filter('content_save_pre', 'wp_filter_post_kses');
    remove_filter('the_content', 'wpautop'); // Disable wpautop temporarily

    $title = sanitize_text_field($request->get_param('title'));
    $content = $request->get_param('content');
    $image_url = esc_url_raw($request->get_param('image_url'));
    $tags = sanitize_text_field($request->get_param('tags'));

    $post_type = get_option('autobloggpt_post_type', 'post');
    $post_status = get_option('autobloggpt_post_status', 'publish');
    $category = get_option('autobloggpt_category', array());
    $post_data = array(
        'post_title' => $title,
        'post_content' => $content,
        'post_status' => $post_status,
        'post_type' => $post_type,
    );

    $post_id = wp_insert_post($post_data);

    if (is_wp_error($post_id)) {
        return new WP_Error('post_creation_failed', 'Failed to create post', array('status' => 500));
    }

    if (!empty($category)) {
        wp_set_post_categories($post_id, (array) $category);
    }

    if (!empty($tags)) {
        $tags_array = explode(',', $tags);
        wp_set_post_tags($post_id, $tags_array);
    }

    if ($image_url) {
        $attachment_id = upload_image_from_url($image_url, $post_id);
        if (!is_wp_error($attachment_id)) {
            set_post_thumbnail($post_id, $attachment_id);
        }
    }
    add_filter('the_content', 'wpautop'); // Re-enable wpautop
    add_filter('content_save_pre', 'wp_filter_post_kses');

    return new WP_REST_Response(array(
        'post_id' => $post_id,
        'status' => 'created',
        'permalink' => get_permalink($post_id),
    ), 200);
}
function autobloggpt_edit_post(WP_REST_Request $request)
{
    remove_filter('content_save_pre', 'wp_filter_post_kses');
    remove_filter('the_content', 'wpautop'); // Disable wpautop temporarily

    $post_id = intval($request->get_param('post_id'));

    if (empty($post_id) || !get_post($post_id)) {
        return new WP_Error('invalid_post_id', 'Invalid post ID', array('status' => 400));
    }

    $title = sanitize_text_field($request->get_param('title'));
    $content = wp_kses_post($request->get_param('content'));
    $image_url = esc_url_raw($request->get_param('image_url'));
    $tags = sanitize_text_field($request->get_param('tags'));
    $category = get_option('autobloggpt_category', array());
    $content = remove_newlines_from_media_tags($content);
    $post_data = array(
        'ID' => $post_id,
        'post_title' => $title,
        'post_content' => $content,
    );

    $updated_post_id = wp_update_post($post_data);

    if (is_wp_error($updated_post_id)) {
        return new WP_Error('post_update_failed', 'Failed to update post', array('status' => 500));
    }

    if (!empty($category)) {
        wp_set_post_categories($updated_post_id, (array) $category);
    }

    if (!empty($tags)) {
        $tags_array = explode(',', $tags);
        wp_set_post_tags($updated_post_id, $tags_array);
    }

    if ($image_url) {
        $attachment_id = upload_image_from_url($image_url, $updated_post_id);
        if (!is_wp_error($attachment_id)) {
            set_post_thumbnail($updated_post_id, $attachment_id);
        }
    }
    add_filter('the_content', 'wpautop'); // Re-enable wpautop
    add_filter('content_save_pre', 'wp_filter_post_kses');
    return new WP_REST_Response(array(
        'post_id' => $updated_post_id,
        'status' => 'updated',
        'permalink' => get_permalink($updated_post_id),
    ), 200);
}
function autobloggpt_delete_post(WP_REST_Request $request)
{
    $post_id = intval($request->get_param('post_id'));

    if (empty($post_id) || !get_post($post_id)) {
        return new WP_Error('invalid_post_id', 'Invalid post ID', array('status' => 400));
    }

    $deleted = wp_delete_post($post_id, true);

    if (!$deleted) {
        return new WP_Error('post_deletion_failed', 'Failed to delete post', array('status' => 500));
    }

    return new WP_REST_Response(array('post_id' => $post_id, 'status' => 'deleted'), 200);
}
function autobloggpt_list_posts(WP_REST_Request $request)
{
    $number_of_posts = intval($request->get_param('number_of_posts')) ?: 10;
    $post_type = get_option('autobloggpt_post_type', 'post');

    $posts = get_posts(array(
        'numberposts' => $number_of_posts,
        'post_type' => $post_type,
        'post_status' => 'publish',
    ));

    if (empty($posts)) {
        return new WP_Error('no_posts_found', 'No posts found', array('status' => 404));
    }

    $post_list = array();
    foreach ($posts as $post) {
        $post_list[] = array(
            'post_id' => $post->ID,
            'title' => get_the_title($post->ID),
            'content' => wp_trim_words($post->post_content, 40, '...'),
            'date' => get_the_date('', $post->ID),
            'author' => get_the_author_meta('display_name', $post->post_author),
            'categories' => wp_get_post_categories($post->ID, array('fields' => 'names')),
            'tags' => wp_get_post_tags($post->ID, array('fields' => 'names')),
            'image_url' => get_the_post_thumbnail_url($post->ID),
        );
    }
    ob_clean();
    return new WP_REST_Response($post_list, 200);
}




/**
 * Helper function to upload an image from a URL and attach it to a post.
 */
function upload_image_from_url($image_url, $post_id)
{
    $image_name = basename($image_url);
    $upload_dir = wp_upload_dir();

    // Download the image file from the URL
    $image_data = file_get_contents($image_url);
    if ($image_data === false) {
        return new WP_Error('image_download_failed', 'Failed to download image', array('status' => 500));
    }

    // Create a file in the upload directory
    $image_file = $upload_dir['path'] . '/' . $image_name;
    file_put_contents($image_file, $image_data);

    // Check the file type and create an attachment
    $wp_filetype = wp_check_filetype($image_name, null);
    $attachment = array(
        'post_mime_type' => $wp_filetype['type'],
        'post_title' => sanitize_file_name($image_name),
        'post_content' => '',
        'post_status' => 'inherit'
    );

    // Insert the attachment into the media library
    $attach_id = wp_insert_attachment($attachment, $image_file, $post_id);
    require_once(ABSPATH . 'wp-admin/includes/image.php');
    $attach_data = wp_generate_attachment_metadata($attach_id, $image_file);
    wp_update_attachment_metadata($attach_id, $attach_data);

    return $attach_id;
}



// @ini_set( 'display_errors', 1);
function detect_hooks_for_url(WP_REST_Request $request)
{
    @ini_set('display_errors', 1);

    // Ensure the URL is a valid string before proceeding
    $url = $request->get_param('url');

    if (!is_string($url) || empty($url)) {
        return new WP_Error('invalid_url', 'The URL provided is invalid or empty.');
    }
    // Temporarily stop output buffer (to prevent any HTML output from interfering)
    ob_start();

    // Store all hooks fired in an array
    $hooks_fired = [];

    // Create a function to capture all hooks
    function log_all_hooks_triggered($hook_name)
    {
        global $hooks_fired;
        $hooks_fired[] = $hook_name; // Store the hook name in the array
    }

    // Add action to capture all hooks fired
    add_action('all', 'log_all_hooks_triggered');

    // Simulate the page load for the given URL (use wp_remote_get)
    $response = wp_remote_get($url);

    // Check if the response is valid or has an error
    if (is_wp_error($response)) {
        $error_message = $response->get_error_message();
        // Handle the error (log, return message, etc.)
        return new WP_Error('request_error', 'Error fetching the URL: ' . $error_message);
    }

    // Now that the request is made, remove the action to stop tracking hooks
    remove_action('all', 'log_all_hooks_triggered');

    // Return or log the fired hooks (to debug or store them)
    return $hooks_fired;
}
function remove_newlines_from_media_tags($content)
{
    // Regex to match both <video> and <audio> tags
    $pattern = '/<(video|audio).*?>.*?<\/\1>/is';

    // Replace callback to process only the matched content
    $content = preg_replace_callback($pattern, function ($matches) {
        // Remove \n and unnecessary whitespace inside the media tag
        $media_content = str_replace("\n", "", $matches[0]);
        $media_content = preg_replace('/\s+/', ' ', $media_content);
        return $media_content;
    }, $content);

    return $content;
}
